@@ -4,7 +4,7 @@
 */
 
 /*
-** $Id: cgi.c,v 1.21 2001/07/18 03:07:52 mrsam Exp $
+** $Id: cgi.c,v 1.22 2001/10/26 04:11:33 mrsam Exp $
 */
 #include	<stdio.h>
 #include	<stdlib.h>
@@ -253,17 +253,17 @@ static const char hex[]="0123456789ABCDEF";
 
 char *cgiurlencode(const char *buf)
 {
-	return (cgiurlencode_common(buf, "\"?;<>&=/:%+#"));
+	return (cgiurlencode_common(buf, "\"?;<>&=/:%@+#"));
 }
 
 char *cgiurlencode_noamp(const char *buf)
 {
-	return (cgiurlencode_common(buf, "\"?<>=/:%+#"));
+	return (cgiurlencode_common(buf, "\"?<>=/:%@+#"));
 }
 
 char *cgiurlencode_noeq(const char *buf)
 {
-	return (cgiurlencode_common(buf, "\"?;<>&/:%+#"));
+	return (cgiurlencode_common(buf, "\"?;<>&/:%@+#"));
 }
 
 void cgi_cleanup()
