@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.
+** Copyright 1998 - 2002 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -38,7 +38,7 @@
 #include	"numlib/numlib.h"
 #include	"tcpd/spipe.h"
 #include	"tcpd/tlsclient.h"
-static const char rcsid[]="$Id: courieresmtpd.c,v 1.31 2001/09/06 03:02:51 mrsam Exp $";
+static const char rcsid[]="$Id: courieresmtpd.c,v 1.33 2002/02/21 00:37:01 mrsam Exp $";
 
 static char helobuf[256];
 static char authuserbuf[256];
@@ -855,6 +855,13 @@ int	authenticated=0;
 	if (chdir(courierdir()))
 		clog_msg_errno();
 
+	if (fcntl(0, F_SETFL, O_NONBLOCK) ||
+	    fcntl(1, F_SETFL, O_NONBLOCK))
+	{
+		perror("fcntl");
+		exit(1);
+	}
+
 	if (strcmp(argv[0], "sendmail") == 0)
 		clog_open_stderr("courieresmtpd");
 		/* We are being called by the sendmail command line wrapper */
@@ -865,20 +872,29 @@ int	authenticated=0;
 
 		clog_open_syslog("courieresmtpd");
 
-		if ((p=getenv("COURIERTLS")) != 0 && *p
-			&& access(p, X_OK) == 0 &&
-			(p=getenv("TLS_CERTFILE")) != 0 && *p &&
-			access(p, R_OK) == 0)
-			starttls=1;
-
-		if ((p=getenv("ESMTP_TLS_REQUIRED")) && atoi(p))
+		if ((p=getenv("ESMTP_TLS")) && atoi(p))
 		{
-			tls_required=1;
-			if (!starttls)
+			tls=1;
+			/* This is the smtps service, and we don't need to
+			 * initialize the STARTTLS feature. */
+		}
+		else
+		{
+			if ((p=getenv("COURIERTLS")) != 0 && *p
+				&& access(p, X_OK) == 0 &&
+				(p=getenv("TLS_CERTFILE")) != 0 && *p &&
+				access(p, R_OK) == 0)
+				starttls=1;
+
+			if ((p=getenv("ESMTP_TLS_REQUIRED")) && atoi(p))
 			{
-				addiovec_error(NULL, "440 TLS not available,  but"
-					" it's required for this connection.");
-				return (1);
+				tls_required=1;
+				if (!starttls)
+				{
+					addiovec_error(NULL, "440 TLS not available,  but"
+						" it's required for this connection.");
+					return (1);
+				}
 			}
 		}
 	}
