@@ -13,6 +13,9 @@
 /* Whether we implement user-extension addressing */
 #undef	LOCAL_EXTENSIONS
 
+/* Allow mail delivery to root */
+#undef	ALLOW_ROOT
+
 #undef	NAME
 
 #undef	testmode
