@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.
+** Copyright 1998 - 2002 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -53,7 +53,7 @@
 #include	"liblock/config.h"
 #include	"liblock/liblock.h"
 
-static const char rcsid[]="$Id: deliver.c,v 1.11 2001/07/30 00:05:47 mrsam Exp $";
+static const char rcsid[]="$Id: deliver.c,v 1.12 2002/02/23 21:21:22 mrsam Exp $";
 
 extern char *local_dotcourier(const char *, const char *, const char **);
 extern char *local_extension();
@@ -287,15 +287,23 @@ const char *curtime;
 			continue;
 		}
 
-		if (fseek(c, 0L, SEEK_SET) == -1)
-		{
-			perror("fseek");
-			exit(EX_TEMPFAIL);
-		}
+		/*
+		** The fd hack is needed for BSD, whose C lib doesn't like
+		** mixing stdio and unistd seek calls.
+		*/
 
 		if (*ctl == '.' || *ctl == '/')
 		{
 		const char *filename=ctl;
+		int fd_hack=dup(fileno(stdin));
+		FILE *fp_hack;
+
+			if (fd_hack < 0 || lseek(fd_hack, 0L, SEEK_SET) < 0 ||
+				(fp_hack=fdopen(fd_hack, "r")) == NULL)
+			{
+				perror("dup");
+				exit(EX_TEMPFAIL);
+			}
 
 			while (*ctl)
 			{
@@ -311,24 +319,35 @@ const char *curtime;
 			}
 
 			if (savemessage(extension, sender, receipient,
-				c, filename,
+				fp_hack, filename,
 				ufromline,
 				dtline,
 				rpline, quota))
 				exit(EX_TEMPFAIL);
+			fclose(fp_hack);
+			close(fd_hack);
 			continue;
 		}
 		if (*ctl == '|')
 		{
 		const char *command=++ctl;
 		int	rc;
+		int fd_hack=dup(fileno(stdin));
+		FILE *fp_hack;
+
+			if (fd_hack < 0 || lseek(fd_hack, 0L, SEEK_SET) < 0 ||
+				(fp_hack=fdopen(fd_hack, "r")) == NULL)
+			{
+				perror("dup");
+				exit(EX_TEMPFAIL);
+			}
 
 			ctl=skip_eol(ctl, 0);
 			while (*command == ' ' || *command == '\t')
 				++command;
 
 			rc=docommand(extension, sender, receipient, defaultext,
-				c, username, userhome, command,
+				fp_hack, username, userhome, command,
 				dtline,
 				rpline,
 				ufromline,
@@ -336,6 +355,8 @@ const char *curtime;
 				recursion_level);
 			if (rc)
 				exit(rc);
+			fclose(fp_hack);
+			close(fd_hack);
 			continue;
 		}
 
