@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: attachments.c,v 1.28 2001/08/22 01:04:08 mrsam Exp $
+** $Id: attachments.c,v 1.31 2002/01/13 18:22:50 mrsam Exp $
 */
 #include	"config.h"
 #include	"sqwebmail.h"
@@ -123,6 +123,10 @@ int	fd2;
 		printf(limiterr, (unsigned long)(max_attach / 1024));
 	}
 
+	if (strcmp(cgi("error"), "makemime") == 0)
+	{
+		printf(getarg("MAKEMIMEERR"), MAKEMIME);
+	}
 	newmsg_hiddenheader("pos", pos);
 	newmsg_hiddenheader("draft", draft);
 	tokennew();
@@ -389,7 +393,7 @@ off_t	dummy;
 	return (0);
 }
 
-static void deleteattach(const char *folder, const char *draft)
+void attach_delete(const char *draft)
 {
 FILE	*fp;
 int	fd2;
@@ -581,9 +585,9 @@ const char *r;
 }
 #endif
 
-static int uploadattach(const char *draft,
-			const char *attpubkey,
-			const char *attprivkey)
+int attach_upload(const char *draft,
+		  const char *attpubkey,
+		  const char *attprivkey)
 {
 	char	*attachfilename;
 	char	*draftfilename;
@@ -742,7 +746,7 @@ static int uploadattach(const char *draft,
 
 		maildir_writemsgstr(newdraftfd, "Content-Type: ");
 		maildir_writemsgstr(newdraftfd, content_type);
-		maildir_writemsgstr(newdraftfd, "; charset=\"");
+		maildir_writemsgstr(newdraftfd, "; format=xdraft; charset=\"");
 		maildir_writemsgstr(newdraftfd, sqwebmail_content_charset);
 		maildir_writemsgstr(newdraftfd, "\"\nContent-Transfer-Encoding: ");
 		maildir_writemsgstr(newdraftfd, content_transfer_encoding);
@@ -945,7 +949,7 @@ static int uploadattach(const char *draft,
 		close(newdraftfd);
 		maildir_deletenewmsg(attachfd, DRAFTS, attachfilename);
 		close(attachfd);
-		return (-1);
+		return (-3);
 	}
 
 	maildir_writemsgstr(newdraftfd, "\n--");
@@ -987,7 +991,7 @@ int	quotaflag=0;
 	{
 		if (!tokencheck())
 		{
-			deleteattach(folder, draft);
+			attach_delete(draft);
 			tokensave();
 		}
 	}
@@ -995,7 +999,7 @@ int	quotaflag=0;
 	{
 		if (!tokencheck())
 		{
-			quotaflag=uploadattach(draft, NULL, NULL);
+			quotaflag=attach_upload(draft, NULL, NULL);
 			tokensave();
 		}
 	}
@@ -1003,7 +1007,7 @@ int	quotaflag=0;
 	{
 		if (!tokencheck())
 		{
-			quotaflag=uploadattach(draft, cgi("pubkey"), NULL);
+			quotaflag=attach_upload(draft, cgi("pubkey"), NULL);
 			tokensave();
 		}
 	}
@@ -1012,7 +1016,7 @@ int	quotaflag=0;
 	{
 		if (!tokencheck())
 		{
-			quotaflag=uploadattach(draft, NULL, cgi("privkey"));
+			quotaflag=attach_upload(draft, NULL, cgi("privkey"));
 			tokensave();
 		}
 	}
@@ -1040,6 +1044,12 @@ int	quotaflag=0;
                   "&form=attachments&pos=%s&draft=%s&error=limits",
                   cgi("pos"), draft);
         }
+	else if (quotaflag == -3)
+	{
+                http_redirect_argss(
+                  "&form=attachments&pos=%s&draft=%s&error=makemime",
+                  cgi("pos"), draft);
+	}
         else
         {
                 http_redirect_argss(
