@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: folder.c,v 1.97 2001/08/22 01:04:08 mrsam Exp $
+** $Id: folder.c,v 1.108 2002/01/13 18:22:50 mrsam Exp $
 */
 #include	<stdio.h>
 #include	<string.h>
@@ -35,6 +35,7 @@
 #include	"filter.h"
 #include	"buf.h"
 #include	"pref.h"
+#include	"newmsg.h"
 #include	"gpg.h"
 
 #if	TIME_WITH_SYS_TIME
@@ -52,12 +53,10 @@
 #endif
 #include	<errno.h>
 
-#define	MIMEGPGFILENAME "mimegpgfilename"
-
 extern void print_attrencodedlen(const char *, size_t, int, FILE *);
 
 extern const char *showsize(unsigned long);
-
+extern void maildir_cleanup();
 extern const char *nonloginscriptptr();
 extern int pref_flagpagesize;
 extern int ishttps();
@@ -224,6 +223,8 @@ int	rc=0;
 		maildir_savefoldermsgs(dir);
 	}
 
+	maildir_cleanup();
+
 	http_redirect_argu(
 		(rc ? "&error=quota&form=folder&pos=%s":"&form=folder&pos=%s"),
 			(unsigned long)pos);
@@ -260,7 +261,7 @@ const char	*qerrmsg;
 
 	printf("<TABLE WIDTH=\"100%%\" CLASS=\"folder-nextprev-background\" BORDER=0 CELLPADDING=0 CELLSPACING=0>");
 	printf("<TR><TD ALIGN=LEFT>");
-	printf("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 CLASS=\"folder-nextprev-buttons\"><TR><TD BGCOLOR=\"#DDDDDD\">");
+	printf("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 CLASS=\"folder-nextprev-buttons\"><TR><TD>");
 	if (morebefore)
 	{
 	size_t	beforepos;
@@ -281,7 +282,7 @@ const char	*qerrmsg;
 
 	printf("</TD><TD ALIGN=RIGHT>\n");
 
-	printf("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 CLASS=\"folder-nextprev-buttons\"><TR><TD BGCOLOR=\"#DDDDDD\">");
+	printf("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 CLASS=\"folder-nextprev-buttons\"><TR><TD>");
 	if (moreafter)
 	{
 		printf("<A HREF=\"");
@@ -296,7 +297,7 @@ const char	*qerrmsg;
 	printf("</TD></TR></TABLE>\n");
 
 	printf("</TD></TR></TABLE>");
-	printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=4><TR BGCOLOR=\"#000080\" CLASS=\"folder-index-header\"><TH ALIGN=CENTER>%s</TH><TH>&nbsp;</TH><TH>%s</TH><TH>%s</TH><TH>%s</TH><TH>%s</TH></TR>\n",
+	printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=4><TR CLASS=\"folder-index-header\"><TH ALIGN=CENTER>%s</TH><TH>&nbsp;</TH><TH>%s</TH><TH>%s</TH><TH>%s</TH><TH>%s</TH></TR>\n",
 
 	       getarg("NUM"),
 	       getarg("DATE"),
@@ -342,8 +343,7 @@ const char	*qerrmsg;
 		if ((q=strrchr(p, '/')) != 0)
 			p=q+1;
 
-		printf("<TR BGCOLOR=\"%s\" CLASS=\"folder-index-bg-%d\"><TD ALIGN=RIGHT>%s%s<font class=\"message-number\">%ld.</font>%s%s</TD><TD><INPUT TYPE=CHECKBOX NAME=\"MOVE-%ld",
-		       (i & 1 ? "#EEEEEE":"#DDDDDD"),
+		printf("<TR CLASS=\"folder-index-bg-%d\"><TD ALIGN=RIGHT>%s%s<font class=\"message-number\">%ld.</font>%s%s</TD><TD><INPUT TYPE=CHECKBOX NAME=\"MOVE-%ld",
 		       (i & 1)+1,
 		       folder_index_entry_start, start_bold,
 		       (long)(i+pos+1),
@@ -392,18 +392,18 @@ const char	*qerrmsg;
 		free(subjs);
 
 		folder_msg_unlink(dir, pos+i, type[0]);
-		printf("%s%s</TD><TD ALIGN=RIGHT>%s%s<font class=\"message-size\">%s</font>%s%s<BR></TD></TR>\n", end_bold, folder_index_entry_end, folder_index_entry_start, start_bold, size, end_bold, folder_index_entry_end);
+		printf("%s%s</TD><TD ALIGN=RIGHT>%s%s<font class=\"message-size\">%s&nbsp;</font>%s%s<BR></TD></TR>\n", end_bold, folder_index_entry_end, folder_index_entry_start, start_bold, size, end_bold, folder_index_entry_end);
 	}
 	if (found)
 	{
-		puts("<TR BGCOLOR=\"#DDDDDD\" CLASS=\"folder-index-bg-1\"><TD COLSPAN=6><HR></TD></TR>");
-		puts("<TR BGCOLOR=\"#EEEEEE\" CLASS=\"folder-index-bg-2\"><TD>&nbsp;</TD>");
+		puts("<TR CLASS=\"folder-index-bg-1\"><TD COLSPAN=6><HR></TD></TR>");
+		puts("<TR CLASS=\"folder-index-bg-2\"><TD>&nbsp;</TD>");
 		printf("<TD COLSPAN=5><INPUT TYPE=CHECKBOX NAME=\"SELECTALL\">&nbsp;%s</TD></TR>",
 			selectalllab);
 	}
 	if (!found && nomsg)
 	{
-		puts("<TR BGCOLOR=\"#DDDDDD\" CLASS=\"folder-index-bg-1\"><TD COLSPAN=6 ALIGN=LEFT><P>");
+		puts("<TR CLASS=\"folder-index-bg-1\"><TD COLSPAN=6 ALIGN=LEFT><P>");
 		puts(nomsg);
 		puts("<BR></TD></TR>");
 	}
@@ -438,7 +438,7 @@ static void folder_msg_link(const char *style,
 	char	*basename=filename ? maildir_basename(filename):NULL;
 
 		output_scriptptrget();
-		printf("&form=newmsg&draft=");
+		printf("&form=open-draft&draft=");
 		output_urlencoded(basename);
 		printf("\">");
 		if (basename)	free(basename);
@@ -659,7 +659,7 @@ void folder_nextprev()
 
 	/* PREV */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\">");
+	printf("<TD CLASS=\"message-menu-button\">");
 
 	if (msg_hasprev)
 	{
@@ -679,7 +679,7 @@ void folder_nextprev()
 
 	/* NEXT */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\">");
+	printf("<TD CLASS=\"message-menu-button\">");
 
 	if (msg_hasnext)
 	{
@@ -699,7 +699,7 @@ void folder_nextprev()
 
 	/* DEL */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\">");
+	printf("<TD CLASS=\"message-menu-button\">");
 	if (msg_type != MSGTYPE_DELETED)
 	{
 		printf("<A HREF=\"");
@@ -720,7 +720,7 @@ void folder_nextprev()
 
 	/* FOLDER */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\"><A HREF=\"");
+	printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
 	output_scriptptrget();
 	printf("&pos=%ld&form=folder\">%s</A></TD>\n",
 		(long)( (msg_pos/pref_flagpagesize)*pref_flagpagesize ),
@@ -728,7 +728,7 @@ void folder_nextprev()
 
 	/* REPLY */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\"><A HREF=\"");
+	printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
 	output_scriptptrget();
 	output_mimegpgfilename();
 	printf("&pos=%ld&reply=1&form=newmsg\">%s</A></TD>\n",
@@ -737,7 +737,7 @@ void folder_nextprev()
 
 	/* REPLY ALL */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\"><A HREF=\"");
+	printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
 	output_scriptptrget();
 	output_mimegpgfilename();
 	printf("&pos=%ld&replyall=1&form=newmsg\">%s</A></TD>\n",
@@ -746,7 +746,7 @@ void folder_nextprev()
 
 	/* REPLY LIST */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\"><A HREF=\"");
+	printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
 	output_scriptptrget();
 	output_mimegpgfilename();
 	printf("&pos=%ld&replylist=1&form=newmsg\">%s</A></TD>\n",
@@ -757,7 +757,7 @@ void folder_nextprev()
 
 	/* FORWARD */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\"><A HREF=\"");
+	printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
 	output_scriptptrget();
 	output_mimegpgfilename();
 	printf("&pos=%ld&forward=1&form=newmsg\">%s</A></TD>\n",
@@ -766,7 +766,7 @@ void folder_nextprev()
 
 	/* FORWARD AS ATTACHMENT*/
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\"><A HREF=\"");
+	printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
 	output_scriptptrget();
 	output_mimegpgfilename();
 	printf("&pos=%ld&forwardatt=1&form=newmsg\">%s</A></TD>\n",
@@ -777,7 +777,7 @@ void folder_nextprev()
 
 	if (!pref_flagfullheaders && !*cgi("fullheaders"))
 	{
-		printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\"><A HREF=\"");
+		printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
 		output_scriptptrget();
 		output_mimegpgfilename();
 		printf("&pos=%ld&form=readmsg&fullheaders=1\">%s</A></TD>\n",
@@ -786,7 +786,7 @@ void folder_nextprev()
 
 	/* PRINT MESSAGE */
 
-	printf("<TD BGCOLOR=\"#DDDDDD\" CLASS=\"message-menu-button\"><A HREF=\"");
+	printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
 	output_scriptptrget();
 	output_mimegpgfilename();
 
@@ -798,14 +798,19 @@ void folder_nextprev()
 	{
 		printf("&pos=%ld&form=print&setcookie=1\" TARGET=\"_BLANK\">%s</A></TD>\n", (long)msg_pos, msg_print);
 	}
-	printf("</TD>");
-	
-	/* end PRINT MESSAGE */
-	
+
+	/* SAVE MESSAGE */
+
+	printf("<TD CLASS=\"message-menu-button\"><A HREF=\"");
+	output_scriptptrget();
+	output_mimegpgfilename();
+
+	printf("&pos=%ld&form=fetch&download=1\">%s</A></TD>", (long)msg_pos,
+	       getarg("SAVEMESSAGE"));
 
     printf("<TD WIDTH=100%%></TD></TR></TABLE></TD><TD ALIGN=RIGHT VALIGN=CENTER>");
 
-	printf("<TABLE BORDER=0 CELLSPACING=4><TR><TD BGCOLOR=\"#666666\" CLASS=\"message-x-of-y\" color=\"#FFFFFF\">&nbsp;");
+	printf("<TABLE BORDER=0 CELLSPACING=4><TR><TD CLASS=\"message-x-of-y\" color=\"#FFFFFF\">&nbsp;");
 	printf(msg_msglab, (int)msg_pos+1, (int)msg_count);
 	printf("&nbsp;</TD></TR></TABLE>");
     printf("</TD></TR></TABLE>\n");
@@ -910,8 +915,8 @@ int	has_shared=0;
 void folder_msgmove()
 {
 	++initnextprevcnt;
-	printf("<TABLE BORDER=0 BGCOLOR=\"#000000\" class=\"box-small-outer\"><TR><TD>\n");
-	printf("<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0><TR><TD BGCOLOR=\"#DDDDDD\" CLASS=\"folder-move-background\">&nbsp;%s&nbsp;<SELECT NAME=list%d>\n", msg_movetolab, initnextprevcnt);
+	printf("<TABLE BORDER=0 class=\"box-small-outer\"><TR><TD>\n");
+	printf("<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0><TR><TD CLASS=\"folder-move-background\">&nbsp;%s&nbsp;<SELECT NAME=list%d>\n", msg_movetolab, initnextprevcnt);
 
 	show_transfer_dest(sqwebmail_folder);
 
@@ -975,6 +980,13 @@ int	rc=0;
 	}
 }
 
+static int is_preview_mode()
+{
+	/* We're in new message window, and we're previewing a draft */
+
+	return (*cgi("showdraft"));
+}
+
 static void showmsgrfc822(FILE *, struct rfc2045 *, struct rfc2045id *);
 static void showmsgrfc822_body(FILE *, struct rfc2045 *, struct rfc2045id *, int);
 
@@ -1039,7 +1051,8 @@ void folder_showmsg(const char *dir, size_t pos)
 	showmsgrfc822_body(fp, rfc, NULL, 0);
 	rfc2045_free(rfc);
 	fclose(fp);
-	maildir_markread(dir, pos);
+	if (*cgi(MIMEGPGFILENAME) == 0)
+		maildir_markread(dir, pos);
 	free(filename);
 }
 
@@ -1107,9 +1120,9 @@ static void dokeyimport(FILE *fp, struct rfc2045 *rfcp, int issecret)
 	int cnt;
 
 	static const char start_str[]=
-		"<TABLE WIDTH=\"100%%\" BORDER=0 BGCOLOR=\"#000000\" class=\"box-outer\"><TR><TD>"
+		"<TABLE WIDTH=\"100%%\" BORDER=0 class=\"box-outer\"><TR><TD>"
 		"<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=4"
-		" BGCOLOR=\"#FFFFFF\" class=\"box-white-outer\"><TR><TD>%s<PRE>\n";
+		" class=\"box-white-outer\"><TR><TD>%s<PRE>\n";
 
 	static const char end_str[]=
 		"</PRE></TD></TR></TABLE></TD></TR></TABLE><BR>\n";
@@ -1424,7 +1437,7 @@ off_t	pos;
 		par="";
 	}
 
-	if (flag)
+	if (flag && !is_preview_mode())
 	{
 		printf("<TR VALIGN=TOP><TD>&nbsp;</TD><TD ALIGN=LEFT VALIGN=TOP>");
 		digestaction(idptr);
@@ -1444,10 +1457,10 @@ off_t	pos;
 		       cgi("pos"));
 		printf("<input type=hidden name=mimegpg value=1>\n");
 
-		printf("<table border=0 bgcolor=\"#000000\" cellpadding=1"
+		printf("<table border=0 cellpadding=1"
 		       " width=\"100%%\" class=\"box-outer\">"
 		       "<tr><td><table width=\"100%%\" border=0 cellspacing=0"
-		       " cellpadding=0 bgcolor=\"#FFFFFF\" class=\"box-white-outer\"><tr><td>");
+		       " cellpadding=0 class=\"box-white-outer\"><tr><td>");
 
 		if ( *cgi(MIMEGPGFILENAME))
 		{
@@ -1517,28 +1530,37 @@ static void showaddressheader_printc(char c, void *p)
 			addr=rfc822_getaddr(sai->a, sai->curindex);
 		}
 
-		printf("<A HREF=\"");
-		output_scriptptrget();
-		printf("&form=quickadd&pos=%s&newname=",
-		       cgi("pos"));
-
-		if (name)
+		if (!is_preview_mode())
 		{
-			output_urlencoded(name);
-			free(name);
-		}
-		printf("&newaddr=");
-		if (addr)
-		{
-			output_urlencoded(addr);
-			free(addr);
-		}
+			printf("<A HREF=\"");
+			output_scriptptrget();
+			printf("&form=quickadd&pos=%s&newname=",
+			       cgi("pos"));
 
-		printf("\" STYLE=\"text-decoration: none\" "
-		       "ONMOUSEOVER=\"window.status='%s'; return true;\" "
-		       "ONMOUSEOUT=\"window.status=''; return true;\" >"
-		       "<FONT CLASS=\"message-rfc822-header-address\">",
-		       msg_add ? msg_add:"");
+			if (name)
+			{
+				char *v=rfc2047_decode_enhanced(name,
+								sqwebmail_content_charset);
+				if (v)
+				{
+					output_urlencoded(v);
+					free(v);
+				}
+				free(name);
+			}
+			printf("&newaddr=");
+			if (addr)
+			{
+				output_urlencoded(addr);
+				free(addr);
+			}
+
+			printf("\" STYLE=\"text-decoration: none\" "
+			       "ONMOUSEOVER=\"window.status='%s'; return true;\" "
+			       "ONMOUSEOUT=\"window.status=''; return true;\" >"
+			       "<FONT CLASS=\"message-rfc822-header-address\">",
+			       msg_add ? msg_add:"");
+		}
 
 		sai->isfirstchar=0;
 	}
@@ -1550,7 +1572,8 @@ static void showaddressheader_printsep(const char *sep, void *p)
 {
 	struct showaddrinfo *sai= (struct showaddrinfo *)p;
 
-	printf("</FONT></A>");
+	printf("</FONT>%s",
+	       is_preview_mode() ? "":"</A>");
 
 	if (sai)
 	{
@@ -1713,13 +1736,18 @@ off_t dummy2;
 	if (strncmp(content_type, "image/", 6) == 0 &&
 		(disposition_s == 0 || strcmp(disposition_s, "attachment")))
 	{
-		printf("<A HREF=\"");
-		output_mimeurl(id, "fetch");
-		printf("\" TARGET=\"_blank\"><IMG SRC=\"");
+		if (!is_preview_mode())
+		{
+			printf("<A HREF=\"");
+			output_mimeurl(id, "fetch");
+			printf("\" TARGET=\"_blank\">");
+		}
+		printf("<IMG SRC=\"");
 		output_mimeurl(id, "fetch");
 		printf("\" ALT=\"Inline picture: ");
 		output_attrencoded(content_type);
-		printf("\"></A>\n");
+		printf("\">%s\n",
+		       is_preview_mode() ? "":"</A>");
 		return;
 	}
 
@@ -1728,8 +1756,8 @@ off_t dummy2;
 	rfc2045_mimepos(rfc, &start_pos, &end_pos, &start_body,
 			&dummy2, &dummy2);
 
-	printf("<TABLE BORDER=0 CELLPADDING=1 CELLSPACING=0 BGCOLOR=\"#000000\" CLASS=\"box-small-outer\"><TR><TD>");
-	printf("<TABLE BORDER=0 CELLPADDING=4 CELLSPACING=0 BGCOLOR=\"#DDDDDD\" CLASS=\"message-download-attachment\"><TR><TD>");
+	printf("<TABLE BORDER=0 CELLPADDING=1 CELLSPACING=0 CLASS=\"box-small-outer\"><TR><TD>");
+	printf("<TABLE BORDER=0 CELLPADDING=4 CELLSPACING=0 CLASS=\"message-download-attachment\"><TR><TD>");
 
 	if (strcmp(cgi("form"), "print") == 0)
 	{
@@ -1742,14 +1770,27 @@ off_t dummy2;
 
 		printf("&nbsp;(%s)</FONT></CENTER>",
 		       showsize(end_pos - start_body));
-		printf("<BR><CENTER><A HREF=\"");
-		output_mimeurl(id, "fetch");
-		printf("\" STYLE=\"text-decoration: none\" TARGET=\"_blank\">"
-		       "%s</A>&nbsp;/&nbsp;<A HREF=\"", getarg("DISPATT"));
+		printf("<BR><CENTER>");
 
-		output_mimeurl(id, "fetch");
-		printf("&download=1\" STYLE=\"text-decoration: none\">"
-		       "%s</A></CENTER>\n", getarg("DOWNATT"));
+		if (!is_preview_mode())
+		{
+			printf("<A HREF=\"");
+			output_mimeurl(id, "fetch");
+			printf("\" STYLE=\"text-decoration: none\" TARGET=\"_blank\">");
+		}
+
+		printf("%s%s&nbsp;/&nbsp;", getarg("DISPATT"),
+		       is_preview_mode() ? "":"</A>");
+
+		if (!is_preview_mode())
+		{
+			printf("<A HREF=\"");
+			output_mimeurl(id, "fetch");
+			printf("&download=1\" STYLE=\"text-decoration: none\">");
+		}
+
+		printf("%s%s</CENTER>\n", getarg("DOWNATT"),
+		       is_preview_mode() ? "":"</A>");
 	}
 
 	printf("</TD></TR></TABLE>\n");
@@ -1759,10 +1800,10 @@ off_t dummy2;
 #if HAVE_SQWEBMAIL_UNICODE
 static void showkey(FILE *fp, struct rfc2045 *rfc, struct rfc2045id *id)
 {
-	printf("<TABLE BORDER=0 CELLPADDING=1 CELLSPACING=0 BGCOLOR=\"#000000\" CLASS=\"box-small-outer\"><TR><TD>");
-	printf("<TABLE BORDER=0 CELLPADDING=4 CELLSPACING=0 BGCOLOR=\"#DDDDDD\" CLASS=\"message-application-pgpkeys\"><TR><TD>");
+	printf("<TABLE BORDER=0 CELLPADDING=8 CELLSPACING=1 CLASS=\"box-small-outer\"><TR><TD>");
+	printf("<TABLE BORDER=0 CELLPADDING=4 CELLSPACING=4 CLASS=\"message-application-pgpkeys\"><TR><TD>");
 
-	if (strcmp(cgi("form"), "print") == 0)
+	if (strcmp(cgi("form"), "print") == 0 || is_preview_mode())
 	{
 		printf("%s", getarg("KEY"));
 	}
@@ -1792,7 +1833,7 @@ static void showkey(FILE *fp, struct rfc2045 *rfc, struct rfc2045id *id)
 	}
 
 	printf("</TD></TR></TABLE>\n");
-	printf("</TD></TR></TABLE>\n");
+	printf("</TD></TR></TABLE>\n<BR>\n");
 }
 #endif
 
@@ -1819,17 +1860,14 @@ char	*s, *t;
 	output_attrencoded(t);
 }
 
-static const char *showmultipartdecoded_start(int status,
-					      const char **styleptr)
+static void showmultipartdecoded_start(int status, const char **styleptr)
 {
-	const char *color= status ? "#EE0000":"#008000";
 	const char *style= status ? "message-gpg-bad":"message-gpg-good";
 
-	printf("<TABLE BORDER=0 CELLPADDING=2 BGCOLOR=\"%s\" CLASS=\"%s\"><TR><TD>"
-	       "<TABLE BORDER=0 BGCOLOR=\"#FFFFFF\" CLASS=\"message-gpg\"><TR><TD>", color, style);
+	printf("<TABLE BORDER=0 CELLPADDING=2 CLASS=\"%s\"><TR><TD>"
+	       "<TABLE BORDER=0 CLASS=\"message-gpg\"><TR><TD>", style);
 	*styleptr=status ? "message-gpg-bad-text":"message-gpg-good-text";
 
-	return (color);
 }
 
 static void showmultipartdecoded_end()
@@ -1854,12 +1892,11 @@ int gpg_status;
 	rfc2045_mimeinfo(rfc, &content_type, &dummy, &dummy);
 
 
-	if (*cgi(MIMEGPGFILENAME) && gpgmime_is_decoded(rfc, &gpg_status))
+	if (*cgi(MIMEGPGFILENAME) && !is_preview_mode() &&
+	    gpgmime_is_decoded(rfc, &gpg_status))
 	{
 		const char *style;
-		const char *color=showmultipartdecoded_start(gpg_status,
-							     &style);
-
+		showmultipartdecoded_start(gpg_status, &style);
 		for (q=rfc->firstpart; q; q=q->next, ++nextpart.idnum)
 		{
 			if (q->isdummy)	continue;
@@ -1867,9 +1904,8 @@ int gpg_status;
 			
 			if (nextpart.idnum == 1)
 			{
-				printf("<BLOCKQUOTE STYLE=\"color: %s\""
-				       " CLASS=\"%s\">",
-				       color, style);
+				printf("<BLOCKQUOTE CLASS=\"%s\">",
+				       style);
 			}
 
 			(*get_handler(q))(fp, q, id);
@@ -2196,6 +2232,20 @@ static void showtextplain(FILE *fp, struct rfc2045 *rfc, struct rfc2045id *id)
 
 	id=id;
 
+	if (is_preview_mode())
+	{
+		const char *cb=rfc2045_getattr(rfc->content_type_attr,
+					       "format");
+		if (cb && strcasecmp(cb, "xdraft") == 0)
+		{
+			preview_start();
+			rfc2045_decodemimesection(fileno(fp), rfc,
+						  &preview_callback, NULL);
+			preview_end();
+			return;
+		}
+	}
+
 	rfc2045_mimeinfo(rfc, &dummy, &dummy, &mime_charset);
 
 	if (strcasecmp(mime_charset, sqwebmail_content_charset) &&
@@ -2382,6 +2432,8 @@ static void add_decoded_link(struct rfc2045 *, const char *, int);
 static char *convertcid(const char *cidurl, void *voidp)
 {
 	struct	rfc2045 *rfc= (struct rfc2045 *)voidp;
+	struct	rfc2045 *savep;
+
 	char	*p;
 	char	*mimeid;
 	char	*q;
@@ -2402,8 +2454,20 @@ static char *convertcid(const char *cidurl, void *voidp)
 			rfc=rfc->parent;
 	}
 
+	savep=rfc;
 	rfc=find_cid(rfc, cidurl);
 
+	if (!rfc)
+		/* Sometimes broken MS software needs to go one step higher */
+	{
+		while ((savep=savep->parent) != NULL)
+		{
+			rfc=find_cid(savep, cidurl);
+			if (rfc)
+				break;
+		}
+	}
+
 	if (!rfc)	/* Not found, punt */
 	{
 	char	*p=malloc(1);
@@ -2564,21 +2628,20 @@ const char *mime_charset, *dummy_s;
 	while (decoded_first)
 	{
 		struct decoded_list *p=decoded_first;
-		const char *color, *style;
+		const char *style;
 
 		struct rfc2045 *q;
 
 		printf("<TR><TD>");
 
-		color=showmultipartdecoded_start(p->status, &style);
+		showmultipartdecoded_start(p->status, &style);
 
 		for (q=p->ptr->firstpart; q; q=q->next)
 		{
 			if (q->isdummy)
 				continue;
 
-			printf("<DIV STYLE=\"color: %s\" CLASS=\"%s\">",
-			       color, style);
+			printf("<DIV CLASS=\"%s\">", style);
 			(*get_handler(q))(fp, q, NULL);
 			printf("</DIV>\n");
 			break;
@@ -2648,7 +2711,7 @@ const char *folderdir;
 	if (strcmp(cgi("foldermsg"), "sent") == 0)
 		folder_err_msg=msg_hasbeensent;
 
-	if (strcmp(cgi("cmd"), "newfolder") == 0)
+	if (*cgi("do.create"))
 	{
 	const char	*newfoldername=cgi("foldername");
 	const char	*newdirname=cgi("dirname");
@@ -2800,14 +2863,14 @@ const char *folderdir;
 	sqwebmail_folder=0;
 	folderdir=cgi("folderdir");
 
-       	printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLPADDING=2 CELLSPACING=0 BGCOLOR=\"#FFFFC0\" class=\"folderlist\">\n");
+       	printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLPADDING=2 CELLSPACING=0 class=\"folderlist\">\n");
 	maildir_readfolders(&folders);
 	if (*folderdir)
 	{
 	char *parentfolder=get_parent_folder(folderdir);
 	size_t	i;
 
-		printf("<TR><TD ALIGN=LEFT COLSPAN=2 BGCOLOR=\"#EEEE80\" class=\"folderparentdir\">%s&lt;&lt;&lt;&nbsp;<A HREF=\"", folders_img);
+		printf("<TR><TD ALIGN=LEFT COLSPAN=2 class=\"folderparentdir\">%s&lt;&lt;&lt;&nbsp;<A HREF=\"", folders_img);
 		output_scriptptrget();
 		printf("&form=folders&folder=INBOX\">");
 		print_safe(name_inbox);
@@ -2892,7 +2955,7 @@ const char *folderdir;
 			s[p-folders[i]]=0;
 
 			img=folders_img;
-			printf("<TR BGCOLOR=\"#EEEE80\" class=\"foldersubdir\"><TD ALIGN=LEFT>%s%s&nbsp;<A HREF=\"", img, pfix);
+			printf("<TR class=\"foldersubdir\"><TD ALIGN=LEFT>%s%s&nbsp;<A HREF=\"", img, pfix);
 			output_scriptptrget();
 			printf("&form=folders&folder=INBOX&folderdir=");
 			output_urlencoded(s);
@@ -2941,7 +3004,7 @@ const char *folderdir;
 			maildir_count(folders[i], &nnew, &nother);
 
 		printf("<TR %s><TD ALIGN=LEFT VALIGN=TOP>",
-			isunsubscribed ? "BGCOLOR=\"#DDDD80\" class=\"folderunsubscribed\"":"");
+			isunsubscribed ? "class=\"folderunsubscribed\"":"");
 
 		printf("%s<INPUT BORDER=0 TYPE=\"radio\" NAME=\"DELETE\" VALUE=\"", img);
 		output_attrencoded(folders[i]);
@@ -3096,7 +3159,7 @@ int	fd;
 	while ((n=fread(buf, 1, sizeof(buf), fp)) > 0)
 		rfc2045_parse(rfc, buf, n);
 
-	part=rfc2045_find(rfc, mimeid);
+	part=*mimeid ? rfc2045_find(rfc, mimeid):rfc;
 	if (!part)	error("Message not found.");
 	rfc2045_mimeinfo(part, &content_type, &dummy, &charset);
 	rfc2045_dispositioninfo(part, &disposition_s, &disposition_name_s,
@@ -3125,25 +3188,48 @@ int	fd;
 
 	rfc2045_mimepos(part, &start_pos, &end_pos, &start_body,
 		&ldummy, &ldummy);
-	if (fseek(fp, start_body, SEEK_SET) < 0)
-	{
-		error("Seek error.");
-		return;
-	}
 
-	rfc2045_cdecode_start(part, &download_func, 0);
-
-	while (start_body < end_pos)
+	if (*mimeid == 0)	/* Download entire message */
 	{
-		cnt=sizeof(buf);
-		if (cnt > end_pos-start_body)
-			cnt=end_pos-start_body;
-		cnt=fread(buf, 1, cnt, fp);
-		if (cnt <= 0)	break;
-		start_body += cnt;
-		rfc2045_cdecode(part, buf, cnt);
+		if (fseek(fp, start_pos, SEEK_SET) < 0)
+		{
+			error("Seek error.");
+			return;
+		}
+
+		while (start_pos < end_pos)
+		{
+			cnt=sizeof(buf);
+			if (cnt > end_pos-start_pos)
+				cnt=end_pos-start_pos;
+			cnt=fread(buf, 1, cnt, fp);
+			if (cnt <= 0)	break;
+			start_pos += cnt;
+			download_func(buf, cnt, NULL);
+		}
+	}
+	else
+	{
+		if (fseek(fp, start_body, SEEK_SET) < 0)
+		{
+			error("Seek error.");
+			return;
+		}
+
+		rfc2045_cdecode_start(part, &download_func, 0);
+
+		while (start_body < end_pos)
+		{
+			cnt=sizeof(buf);
+			if (cnt > end_pos-start_body)
+				cnt=end_pos-start_body;
+			cnt=fread(buf, 1, cnt, fp);
+			if (cnt <= 0)	break;
+			start_body += cnt;
+			rfc2045_cdecode(part, buf, cnt);
+		}
+		rfc2045_cdecode_end(part);
 	}
-	rfc2045_cdecode_end(part);
 	fclose(fp);
 }
 
