@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: gpg.c,v 1.7 2001/08/22 01:04:08 mrsam Exp $
+** $Id: gpg.c,v 1.8 2001/10/01 22:52:25 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"config.h"
@@ -92,7 +92,7 @@ static int show_key(const char *fingerprint, const char *shortname,
 {
 	struct listinfo *li=(struct listinfo *)gli->voidarg;
 
-	printf("<TR VALIGN=MIDDLE BGCOLOR=\"#D0D0D0\" CLASS=\"%s\"><TD>"
+	printf("<TR VALIGN=MIDDLE CLASS=\"%s\"><TD>"
 	       "<input type=radio name=%s value=\"",
 	       li->issecret ? "gpgseckey":"gpgpubkey",
 	       li->issecret ? "seckeyname":"pubkeyname");
@@ -147,14 +147,14 @@ static void listpubsec(int flag,
 
 void gpglistpub()
 {
-	printf("<table width=\"100%%\" bgcolor=\"#000000\" border=0 cellspacing=2 cellpadding=0 class=\"gpgpubkeys\">");
+	printf("<table width=\"100%%\" border=0 cellspacing=2 cellpadding=0 class=\"gpgpubkeys\">");
 	listpubsec(0, show_key, NULL);
 	printf("</table>");
 }
 
 void gpglistsec()
 {
-	printf("<table width=\"100%%\" bgcolor=\"#000000\" border=0 cellspacing=2 cellpadding=0 class=\"gpgseckeys\">");
+	printf("<table width=\"100%%\" border=0 cellspacing=2 cellpadding=0 class=\"gpgseckeys\">");
 	listpubsec(1, show_key, NULL);
 	printf("</table>");
 }
